/********************************************************************/
/*                                                                  */
/* Program name: SAMPMQDX.C                                         */
/*                                                                  */
/* Description: Source file for the three routines used by the      */
/*              Dialer Exit sample DLL.                             */
/*                                                                  */
/* Statement:     Licensed Materials - Property of IBM              */
/*                                                                  */
/*                5622-960 (C) Copyright IBM Corp. 1996             */
/*                                                                  */
/*                U.S. Government Users Restricted Rights - use,    */
/*                duplication or disclosure restricted by GSA ADP   */
/*                Schedule Contract with IBM Corp.                  */
/*                                                                  */
/********************************************************************/
#include <windows.h>                         /* Windows header file */
#include "sampmqdx.h"            /* sample dialer exit include file */



/********************************************************************/
/*                                                                  */
/* Global Variables                                                 */
/*                                                                  */
/********************************************************************/
FARPROC  vlpfnConnectionNowUp = 0L;
FARPROC  vlpfnConnectionNowDown = 0L;


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_Initialize                                   */
/*                                                                  */
/* Description: Function to initialize the dialer interface         */
/*                                                                  */
/* Parameters in:  Window handle of the Dialer Extension Daemon and */
/*                 Addresses of call-back functions to be called    */
/*                 when the connection is made or dropped           */
/*                                                                  */
/* Returns:                                                         */
/*   TRUE  : Interface initialized successfully                     */
/*   FALSE : Interface could not be initialized successfully        */
/*                                                                  */
/********************************************************************/
BOOL _loadds _far MQDX_Initialize( HWND    hwndDialerExtension,
                                   FARPROC lpfnConnectionNowUp,
                                   FARPROC lpfnConnectionNowDown )
{
    /*
     * Remember the callback addresses
     */
    vlpfnConnectionNowUp   = lpfnConnectionNowUp;
    vlpfnConnectionNowDown = lpfnConnectionNowDown;

    return(TRUE);
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_WhichFeaturesSupported                       */
/*                                                                  */
/* Description: Function to return which features this dialer       */
/*              interface supports                                  */
/*                                                                  */
/* Returns:                                                         */
/*   Combination of the following bit-flags:                        */
/*      CALLBACK_ON_CONNECTION_CHANGE                               */
/*      SERVER_INITIATED_CALLS                                      */
/*                                                                  */
/********************************************************************/
LONG _loadds _far MQDX_WhichFeaturesSupported( void )
{
    LONG   lFlags;
    /*
     * Set which features we support
     */
    lFlags = 0;
    return(lFlags);
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_StartDialer                                  */
/*                                                                  */
/* Description: Function to start the dialer program.               */
/*                                                                  */
/* Returns:                                                         */
/*   TRUE  : Dialer started successfully.                           */
/*   FALSE : Dialer could not be started.                           */
/*                                                                  */
/********************************************************************/
UINT _loadds _far MQDX_StartDialer( void )
{
   UINT   RC;
   RC = WinExec( DIALER_FILE_NAME, SW_SHOW );
   if (RC < 32) return(RC+100);
   else return(0);
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_IsDialerConnectionActive                     */
/*                                                                  */
/* Description: Function to check whether or not the dier           */
/*              connection is active.                               */
/*                                                                  */
/* Returns:                                                         */
/*   TRUE  : Dialer connection active.                              */
/*   FALSE : Dialer connection inactive.                            */
/*                                                                  */
/********************************************************************/
BOOL _loadds _far MQDX_IsDialerConnectionActive( void )
{
   return FALSE;
}

/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_StopDialer                                   */
/*                                                                  */
/* Description: Function to pass control back to the dialer program */
/*              enabling it to be stopped by the user.              */
/*                                                                  */
/* Returns:                                                         */
/*   None.                                                          */
/*                                                                  */
/********************************************************************/
UINT _loadds _far MQDX_StopDialer( void )
{
   return( 0 );
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_Terminate                                    */
/*                                                                  */
/* Description: Function to terminate the dialer interface.         */
/*                                                                  */
/* Returns:                                                         */
/*   None.                                                          */
/*                                                                  */
/********************************************************************/
void _loadds _far MQDX_Terminate( void )
{
    /*
     * Remember the callback addresses are no longer valid
     */
    vlpfnConnectionNowUp   = 0L;
    vlpfnConnectionNowDown = 0L;
}
