/********************************************************************/
/*    ...                                                           */
/* Program name: WIN16RAS.C                                         */
/*                                                                  */
/* Description:  Source file for the Windows 16 RAS Dialer          */
/*               Extension.                                         */
/*                                                                  */
/* Requirements:  Requires Microsoft RAS API support file           */
/*                                                                  */
/*                      ras.h                                       */
/*                      raserror.h                                  */
/*                      rasapi16.lib                                */
/*                                                                  */
/*                These are obtainable from the Windows SDK         */
/*                on the Microsoft Developer Network                */
/*                                                                  */
/* Statement:     Licensed Materials - Property of IBM              */
/*                                                                  */
/*                5622-960 (C) Copyright IBM Corp. 1996             */
/*                                                                  */
/*                U.S. Government Users Restricted Rights - use,    */
/*                duplication or disclosure restricted by GSA ADP   */
/*                Schedule Contract with IBM Corp.                  */
/*                                                                  */
/********************************************************************/

#include <windows.h>             // Windows header file
#include <string.h>              // string functions
#include <stdlib.h>              // standard library functions
#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>              // memory functions
#include <ras.h>                 // Windows RAS header file
#include <raserror.h>            // Windows RAS errors header file
#include <win16ras.h>            // sample dialer exit include file



/********************************************************************/
/*                                                                  */
/* Function Name: LibMain                                           */
/*                                                                  */
/* Description: Function to initialize WIN16RAS.DLL                 */
/*                                                                  */
/* Parameters                                                       */
/*   hInstance   : Library instance handle                          */
/*   wDataSeg    : Library data segment                             */
/*   cbHeapSize  : Default heapszize                                */
/*   lpszCmdLine : Command-lone arguments                           */
/*                                                                  */
/* Returns:                                                         */
/*   TRUE  : DLL initialized successfully                           */
/*   FALSE : DLL could not be initialized successfully              */
/*                                                                  */
/********************************************************************/

int CALLBACK LibMain( HINSTANCE hInstance, WORD wDataSeg,
                      WORD cbHeapSize, LPSTR lpszCmdLine )
{
  WNDCLASS wc;               /* window class for information window */
  //
  //  Get the background that we will be using in animated icon etc
  //

  vhbrushDlgBkgnd = CreateSolidBrush( GetSysColor( COLOR_INACTIVEBORDER ) );


  // fill in a window class structure with parameters
  // that describe the Info window.

  wc.style         = NULL;             /* Class style               */
  wc.lpfnWndProc   = InfoWinProc;      /* WinProc address    */
  wc.cbClsExtra    = 0;                /* No per-class extra data   */
  wc.cbWndExtra    = 4;                /* No per-window extra data  */
  wc.hInstance     = hInstance;        /* Appl that owns the class  */
  wc.hIcon         = NULL;
  wc.hCursor       = NULL;
  wc.hbrBackground = vhbrushDlgBkgnd;
  wc.lpszMenuName  = NULL;             /* Name of menu in .RC file  */
  wc.lpszClassName = INFO_CLASSNAME;   /* Name used in CreateWindow */

  /* Register the window class and return if fails
   */
  if( !RegisterClass( &wc) )
  {
    /* warn user of problem and get out */
    MessageBeep( MB_ICONEXCLAMATION );
    return 0;
  }

  wc.lpfnWndProc   = IconWinProc;      /* InfoWInProcAddress */
  wc.lpszClassName = INFO2_CLASSNAME;  /* Name used in CreateWindow */
  wc.hbrBackground = NULL;             // Infowin proc to process background


  if( !RegisterClass( &wc) )
  {
    /* warn user of problem and get out */
    MessageBeep( MB_ICONEXCLAMATION );
    return 0;
  }
  else
  {

    /* save instance handle */
    vhInstance = hInstance;

    /* load icons for icon window */
    vahIcon[0] = LoadIcon( vhInstance,  "ICON_APPL" );
    vahIcon[1] = LoadIcon( vhInstance, "ICON_DIAL1" );
    vahIcon[2] = LoadIcon( vhInstance, "ICON_DIAL2" );
    vahIcon[3] = LoadIcon( vhInstance, "ICON_DIAL3" );
    vahIcon[4] = LoadIcon( vhInstance, "ICON_DIAL4" );
    vahIcon[5] = vahIcon[3];
    vahIcon[6] = vahIcon[2];
    vahIcon[7] = vahIcon[1];

    /* create colour brushes, pens and references */
    vhbrushDlgBkgnd = CreateSolidBrush( GetSysColor( COLOR_INACTIVEBORDER ) );
    vclrrefDlgTextFrgnd = GetSysColor( COLOR_WINDOWTEXT );
    vclrrefDlgTextBkgnd = GetSysColor( COLOR_INACTIVEBORDER );
    vhpenEditTopBorder = CreatePen( PS_SOLID, 1, PALETTERGB(128, 128, 128) );

    /* load commonly-used strings (e.g. application title) */
    LoadString( vhInstance, IDS_DEFAULT_INFO_TEXT, (LPSTR)vszInfoText, INFO_SIZE );
    LoadString( vhInstance, IDS_APP_NAME, (LPSTR)vszAppName, TITLE_SIZE );
    return 1;
  }
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_Initialize                                   */
/*                                                                  */
/* Description: Function to initialize the dialer interface         */
/*                                                                  */
/* Parameters                                                       */
/*   hwnd                  : MQWIN window handle                    */
/*   lpfnConnectionNowUp   : Connection made callback address       */
/*   lpfnConnectionNowDown : Connection dropped callback address    */
/*                                                                  */
/*                                                                  */
/* Returns:                                                         */
/*   TRUE  : Interface initialized successfully                     */
/*   FALSE : Interface could not be initialized successfully        */
/*                                                                  */
/********************************************************************/

BOOL _loadds _far MQDX_Initialize( HWND hwnd, FARPROC lpfnConnectionNowUp,
                                   FARPROC lpfnConnectionNowDown )
{
  // remember the callback addresses, and calling window handle

  vlpfnConnectionNowUp   = lpfnConnectionNowUp;
  vlpfnConnectionNowDown = lpfnConnectionNowDown;
  vhwnd = hwnd;
  vhRasConn = NULL;
  vNoExitHangUp=FALSE;

  return TRUE;
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_WhichFeaturesSupported                       */
/*                                                                  */
/* Description: Function to return which features this dialer       */
/*              interface supports                                  */
/*                                                                  */
/* Returns:                                                         */
/*   Combination of the following bit-flags:                        */
/*      CALLBACK_ON_CONNECTION_CHANGE                               */
/*      SERVER_INITIATED_CALLS                                      */
/*                                                                  */
/********************************************************************/

LONG _loadds _far MQDX_WhichFeaturesSupported( void )
{

  // return  which features we support

  return ( CALLBACK_ON_CONNECTION_CHANGE );
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_StartDialer                                  */
/*                                                                  */
/* Description: Function to start the dialer program.               */
/*                                                                  */
/* Returns:                                                         */
/*   0   : Dialer started successfully.                             */
/*   > 0 : Dialer could not be started.                             */
/*                                                                  */
/********************************************************************/
UINT _loadds _far MQDX_StartDialer( void )
{
  LPRASENTRYNAME lpRasEntry;    // pointer to RAS Entry information
  LPRASENTRYNAME lpTemp;        // temp pointer to RAS Entry information
  DWORD cbBuf;                  // size of buffer allocated to hold RAS Entries
  DWORD cEntry;                 // size of buffer required to hold RAS Entries
  DWORD dwRet;                  // RAS return code
  char szMessage[256];          // buffer to hold RAS error messages
  RASENTRIES re;                // RAS entries information

  //-----------------------------------------------------------------------//
  // Restriction:                                                          //
  //-----------------------------------------------------------------------//
  // Here we want to check if the MQW user wants to break an existing      //
  // connection on continue to use it.                                     //
  // Note this code assumes we make only ONE RasConnection at a time       //
  // Risk here if more than once connection made on system of picking the  //
  // wrong one                                                             //
  //                                                                       //
  //-----------------------------------------------------------------------//

   if (SetConnectionHandle() !=FALSE)  // Get connection handle if one made
   {
    SetFocus(vhwndConnect);            // set focus on currently open dialogue

    //
    // Here we could walk through each connection in turn if there is
    // more than one serial link in use, see SetConnectionHandle
    //
    sprintf(szMessage,"Use current connection to %s?",vszEntryName);

    if (MessageBox( (HWND)NULL, szMessage,
         vszAppName, MB_ICONQUESTION | MB_YESNO ) == IDYES)
     {
           vNoExitHangUp=TRUE;
           return(0);
     }
     else
     {
     vNoExitHangUp=FALSE;
     HangUp(vhRasConn);         // NO , so hang up
     }
   }

  // Allocate buffer to hold RAS Entries (one entry only) and
  // enumerate the entries configured


  cbBuf = sizeof( RASENTRYNAME );
  if( (lpRasEntry = ( LPRASENTRYNAME )_fmalloc((UINT)cbBuf)) != NULL )
  {
    lpRasEntry->dwSize = sizeof(RASENTRYNAME);
    dwRet = RasEnumEntries( NULL, NULL, lpRasEntry, &cbBuf, &cEntry );

    // if buffer too small, reallocate buffer using the size returned on the
    // REE() call and retry

    if ( dwRet == ERROR_BUFFER_TOO_SMALL )
    {
      if( (lpTemp = ( LPRASENTRYNAME )_frealloc( lpRasEntry, (UINT)cbBuf )) != NULL )
      {
        lpRasEntry = lpTemp;
        dwRet = RasEnumEntries( NULL, NULL, lpRasEntry, &cbBuf, &cEntry );
      }
      else
      {
        dwRet = ERROR_NOT_ENOUGH_MEMORY;
      }
    }

    // for all errors, get error string and report error

    if( dwRet != 0 )
    {

      if( RasGetErrorString( (UINT)dwRet, szMessage, 256 ) != 0 )
      {
        wsprintf( (LPSTR)szMessage, "Undefined RasEnumEntries() Error." );
      }
        MessageBox( (HWND)NULL, (LPSTR)szMessage, vszAppName, MB_OK | MB_ICONSTOP );
    }

    // RAS Entries found, so call the Select... dialog to enable the
    // user to choose a connection - pass the count and entry details to
    // the dialog box

    else
    {
      re.cEntry = cEntry;
      re.lpRasEntry = lpRasEntry;
      dwRet = (DWORD)DialogBoxParam( vhInstance, "IDD_SELECT", vhwnd,
                                     SelectDlgProc, (LPARAM)&re );
    }

    // free memory allocated to hold RAS Entries

    _ffree( lpRasEntry );
  }

  // return zero if connection was successful, RAS return code otherwise

  return (UINT)( dwRet ? dwRet : 0 );
}


/********************************************************************/
/*                                                                  */
/* Function Name: Hangup                                            */
/*                                                                  */
/* Description: Calls RasHangup unless a connection was already     */
/*              made prior to starting the link                     */
/*                                                                  */
/*                                                                  */
/********************************************************************/

VOID HangUp(HRASCONN hRas)
{
  RASCONNSTATUS rasStatus;                  /* RAS Status structure */

  /* get connection status */

  rasStatus.dwError = 0;                    // set non zero status  */
  rasStatus.dwSize = sizeof(RASCONNSTATUS);


  if(hRas !=NULL  && vNoExitHangUp==FALSE)
  {

  if ( RasGetConnectStatus( hRas, &rasStatus ) != ERROR_INVALID_HANDLE )

   {
     RasHangUp( hRas );          // returns 0 if ok
   }                             //
  }

}


/********************************************************************/
/*                                                                  */
/* Function Name: SetConnectionHandle                               */
/*                                                                  */
/* Description: Function to set connection handle if not already set*/
/*              connection is active.                               */
/*                                                                  */
/* Returns:                                                         */
/*   NUMBER OF ACTIVE CONNECTIONS if handle is set                  */
/*   FALSE : Dialer connection handle not set/ inactive link        */
/*                                                                  */
/********************************************************************/

UINT  SetConnectionHandle(void )  // return number of connections if set

{

  DWORD    cbBuf;
  RASCONN  *lpRasConns;
  RASCONN  *lpTemp;
  DWORD    cConnections;
  DWORD dwRet;                  // RAS return code
  char  szMessage[256];         // buffer to hold RAS error messages

  // See if any connection is already active ,set handle  //



  vhRasConn=NULL; // reset the handle

  cbBuf = sizeof( RASCONN );

  if( (lpRasConns = ( RASCONN*)_fmalloc((UINT)cbBuf)) != NULL )
  {

    lpRasConns->dwSize = sizeof(RASCONN);

    dwRet = RasEnumConnections(lpRasConns,&cbBuf,&cConnections);

    // if buffer too small, reallocate buffer using the size returned on the
    // REE() call and retry
    //

    if ( dwRet == ERROR_BUFFER_TOO_SMALL )
    {
      if( (lpTemp = ( LPRASCONN )_frealloc( lpRasConns, (UINT)cbBuf )) != NULL )
      {
        lpRasConns = lpTemp;

        dwRet = RasEnumConnections( lpRasConns,&cbBuf, &cConnections );
      }
      else
      {
        dwRet = ERROR_NOT_ENOUGH_MEMORY;
      }
    }

    /* for all errors, get error string and report error */

    if( dwRet != 0 )
    {
      if( RasGetErrorString( (UINT)dwRet, szMessage, 256 ) != 0 )
      {
        wsprintf( (LPSTR)szMessage, "Undefined RasEnumConnections() Error." );
      }
      MessageBox( (HWND)NULL, (LPSTR)szMessage, vszAppName, MB_OK | MB_ICONSTOP );
    }
    else        // Here if we got some connection status back
    {


    if (cConnections)           // copy entry name for later use
       {

         vhRasConn =  lpRasConns->hrasconn;
         strcpy(vszEntryName,lpRasConns->szEntryName);

       }

    _ffree( lpRasConns );
    }
  }
  return(vhRasConn!=NULL ? (UINT)cConnections : FALSE);
}

/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_IsDialerConnectionActive                     */
/*                                                                  */
/* Description: Function to check whether or not the dier           */
/*              connection is active.                               */
/*                                                                  */
/* Returns:                                                         */
/*   TRUE  : Dialer connection active.                              */
/*   FALSE : Dialer connection inactive.                            */
/*                                                                  */
/********************************************************************/
BOOL _loadds _far MQDX_IsDialerConnectionActive( void )
{
  RASCONNSTATUS rasStatus;                  // RAS Status structure
  DWORD dwRet;                              // RAS return code

  /* get connection status */

  rasStatus.dwError = 65384;                // set non zero status  */
  rasStatus.dwSize = sizeof(RASCONNSTATUS);

  dwRet = RasGetConnectStatus( vhRasConn, &rasStatus );


  // return connection inactive if either the API call fails
  // or the API call reported an error
  //
  // This code allows up to pick up the new handle from an externally
  //  (to this dll) connected line. This does assume that we have only ONE
  // connection made on the system at any ONE time.

  if( dwRet )       // API call may fail if handle vhrasconn is invalid
  {                 // e.g external RasHangup() call dropped the line

   if (SetConnectionHandle() != FALSE)
    {
      rasStatus.dwError = 65384;                // set non zero status  */
      rasStatus.rasconnstate = 0;               // set  zero status     */
      dwRet = RasGetConnectStatus(vhRasConn,&rasStatus);
    }

  }

 if(!dwRet)     // OK return from RasGetConnectStatus
  {
   if( rasStatus.rasconnstate == RASCS_Connected  && rasStatus.dwError ==0 )
       return TRUE;
  }

  return FALSE;

}

/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_StopDialer                                   */
/*                                                                  */
/* Description: Function to pass control back to the dialer program */
/*              enabling it to be stopped by the user.              */
/*                                                                  */
/* Returns:  0 Always !                                             */
/*                                                                  */
/********************************************************************/
UINT _loadds _far MQDX_StopDialer( void )
{
  if(vhRasConn!=NULL)
  {
   HangUp(vhRasConn);
  }
  else
  {
   if (SetConnectionHandle() != FALSE)    // if we had a previous connection
    {
      HangUp(vhRasConn);
    }
  }
  vhwndConnect = 0;                       // serialisation flag
  return 0;
}


/********************************************************************/
/*                                                                  */
/* Function Name: MQDX_Terminate                                    */
/*                                                                  */
/* Description: Function to terminate the dialer interface.         */
/*                                                                  */
/* Returns:                                                         */
/*   None.                                                          */
/*                                                                  */
/********************************************************************/
void _loadds _far MQDX_Terminate( void )
{
    //
    // Remember the callback addresses are no longer valid
    //
    vlpfnConnectionNowUp   = 0L;
    vlpfnConnectionNowDown = 0L;
    vhwndConnect = 0;
}

/**********************************************************************/
/*                                                                    */
/* Function Name: SelectDlgProc                                       */
/*                                                                    */
/* Description: Select RAS Connection dialog procedure.               */
/*                                                                    */
/* Input Parameters:                                                  */
/*                                                                    */
/*   hdlg     : Window handle.                                        */
/*   iMessage : Message identifier.                                   */
/*   wParam   : 16 bits of additional message dependent               */
/*              information.                                          */
/*   lParam   : 32 bits of additional message dependent               */
/*              information.                                          */
/*                                                                    */
/* InOut Parameters:                                                  */
/*                                                                    */
/*   None.                                                            */
/*                                                                    */
/* Output Parameters:                                                 */
/*                                                                    */
/*   BOOL     : dependent on message type                             */
/*                                                                    */
/**********************************************************************/
BOOL __export CALLBACK SelectDlgProc( HWND hdlg, UINT message,
                                     WPARAM wParam, LPARAM lParam )
{
  RECT rect;                               // window update rectangle
  UINT ndx;                                // loop counter
  PRASENTRIES lpre;                        // pointer to RAS Entries structure
  RASDIALPARAMS rdParams;                  // dial parameters structure
  char szEntry[RAS_MaxEntryName+1];        // RAS connection entry
  DWORD dwRet;                             // RAS return code
  char  szBuf[256];                        // general purpose buffer
  char  szBuf1[20];                        // general purpose buffer
  RASCONNSTATUS rasStatus;                 // RAS Status structure

  // handle messages sent from RAS first

  if( message == vuMessageRAS )
  {
    // if error:
    if( lParam )
    {
      // Stop animated icon

      if(vtimerDial) KillTimer( hdlg, vtimerDial );
      vndxIcon = 0;
      InvalidateRect( vhwndIconLine, NULL, TRUE );
      UpdateWindow( vhwndIconLine );

      //-------------------------------------------------------------------//
      // report RAS error, hang-up connection and return to the Select...  //
      // dialog                                                            //
      //-------------------------------------------------------------------//

      if( RasGetErrorString( (UINT)lParam, szBuf, 256 ) != 0 )
       {
          wsprintf( (LPSTR)szBuf, "Undefined RAS Dial Error %d",lParam );
       }

      wsprintf( szBuf1, "code = %d)",lParam);
      strcat(szBuf,szBuf1);
      MessageBox(hdlg, (LPSTR)szBuf, vszAppName, MB_OK | MB_ICONSTOP );



      HangUp(vhRasConn);         // Hang up / clean up RAS

      SetInfoText( hdlg, vszInfoText );

      EnableWindow( GetDlgItem( hdlg, IDC_CONNECT ), TRUE );
    }

      //------------------------------------//
      // if connection is successful        //
      //------------------------------------//


    else
    {

      if( RASCS_DONE & wParam )
      /* return control back to MQWIN */
      EndDialog( hdlg, 0 );
    }
    return FALSE;
  }

  switch( message )
  {
    case WM_INITDIALOG:

      vWaitForHangUp=FALSE;

      /* get pointer to RAS Entries inforamtion */
      lpre = (PRASENTRIES)lParam;

      /* get commonly used window handles */
      vhwndGroup1 = GetDlgItem( hdlg, IDC_GROUP1 );
      vhwndRASList = GetDlgItem( hdlg, IDC_RASLIST );
      vhwndIconLine = GetDlgItem( hdlg, IDC_INFOLINE2);

      // subclass the group boxes to paint them nicely

      vDefGroupWinProc = (WNDPROC)SetWindowLong( vhwndGroup1, (int)GWL_WNDPROC,
                                                 (LONG)GroupWinProc );
      // set dialog title

      SetWindowText( hdlg, vszAppName );

      // display the default info string (i.e. Select RAS connection...)

      SetInfoText( hdlg, vszInfoText );

      // populate listbox with all entries returned from RAS

      for( ndx = 0; ndx < lpre->cEntry; ndx++ )
        SendDlgItemMessage( hdlg, IDC_RASLIST, LB_ADDSTRING,
                            0, (LPARAM)(LPCSTR)lpre->lpRasEntry[ndx].szEntryName );

      // if one or more entries found, then select first item in the
      // list and enable the Connect pushbutton, otherwise get out

      if( ndx > 0 )
      {
        SendDlgItemMessage( hdlg, IDC_RASLIST, LB_SETCURSEL, 0, 0 );
        EnableWindow( GetDlgItem( hdlg, IDC_CONNECT ), TRUE );
      }
      else
        EnableWindow( GetDlgItem( hdlg, IDC_CONNECT ), FALSE );


      vhwndConnect = hdlg;  // Mark as initialised by this session
      break;


                            // redraw dialog background
    case WM_ERASEBKGND:
      GetClientRect(hdlg, &rect);
      FillRect( (HDC)wParam, &rect, vhbrushDlgBkgnd );
      return TRUE;
      break;

                           // set dialog control(buttons/statics) colours
    case WM_CTLCOLOR:
      switch (HIWORD(lParam))
      {
        case CTLCOLOR_STATIC:
          SetTextColor( (HDC)wParam, vclrrefDlgTextFrgnd );
          SetBkColor( (HDC)wParam, vclrrefDlgTextBkgnd );
          return vhbrushDlgBkgnd;
          break;

        case CTLCOLOR_BTN:
          return vhbrushDlgBkgnd;
          break;
      }
      break;

    case WM_COMMAND:
      switch( wParam )
      {
                              // start connection
        case IDC_CONNECT:

          //------------------------------------------------------------//
          // get the selected RAS connection and change the information //
          // line to inform the user which connection is being dialed   //
          //------------------------------------------------------------//

          ndx = (UINT)SendMessage( vhwndRASList, LB_GETCURSEL, 0,  0L );
          SendMessage( vhwndRASList, LB_GETTEXT, ndx, (LPARAM)(LPSTR)szEntry );

          // You may want to provide NLS support here !

          wsprintf( (LPSTR) szBuf, "Dialing %s...", (LPSTR)szEntry );

          SetInfoText( hdlg, (LPSTR)szBuf );

          // setup RASDial() parameters

          rdParams.dwSize = sizeof(RASDIALPARAMS);
          lstrcpy( rdParams.szEntryName, (LPSTR)szEntry );
          rdParams.szPhoneNumber[0] = '\0';
          rdParams.szCallbackNumber[0] = '*';
          rdParams.szCallbackNumber[0] = '\0';
          rdParams.szUserName[0] = '\0';
          rdParams.szPassword[0] = '\0';
          rdParams.szDomain[0] = '*';
          rdParams.szDomain[1] = '\0';
          vhRasConn = NULL;

            // get RAS event notify message identifier

          vuMessageRAS = RegisterWindowMessage( RASDIALEVENT );

          /* make connection */
          dwRet = RasDial( NULL, NULL, &rdParams, NULL, hdlg, &vhRasConn );

          vtimerDial=0;

          // if error - get RAS error message and display it

          if( dwRet )
          {
            if( RasGetErrorString( (UINT)dwRet, (LPSTR)szBuf, 256 ) != 0 )
            {
             wsprintf( (LPSTR)szBuf, "Undefined RASDial() Error (%ld).", dwRet );
            }
            MessageBox( hdlg, (LPSTR)szBuf, vszAppName, MB_OK | MB_ICONSTOP );
          }
          else
          {
          //------------------------------------------------------------//
          // Grey out connect button and disable it                     //
          //------------------------------------------------------------//
          EnableWindow( GetDlgItem( hdlg, IDC_CONNECT ), FALSE );
          vtimerDial = SetTimer( hdlg, (UINT)VI_DIAL_TIMERID,
                                 (UINT)(VI_DIAL_TIMEOUT), (TIMERPROC)NULL );
          }
          break;

        case IDCANCEL:

          //---------------------------------------------------------------//
          // Select... dialog is being dismissed, but hang-up just in case //
          // a connection was in progress and kill the dial timer          //
          //---------------------------------------------------------------//

          HangUp(vhRasConn);
          //
          //  use timer to wait for hangup complete before killing dialogue
          //
          //
          vWaitForHangUp = TRUE;
          if(vtimerDial) KillTimer( hdlg, vtimerDial );
          EndDialog( hdlg, TRUE ) ;
          vhwndConnect = 0;
          return TRUE;
      }
      break;

    case WM_TIMER:

      if(vWaitForHangUp)
      {
        vWaitForHangUp++;

       // get connection status

       rasStatus.dwError = 0;                    // set non zero status  */
       rasStatus.dwSize = sizeof(RASCONNSTATUS);
       if ( RasGetConnectStatus( vhRasConn, &rasStatus ) == ERROR_INVALID_HANDLE )
       {
        vhwndConnect = 0;
        vWaitForHangUp=FALSE;
        if(vtimerDial) KillTimer( hdlg, vtimerDial );
        EndDialog( hdlg, TRUE ) ;
       }
      }
      else
      {
      /* update the animated dial timer */
      (vndxIcon == 7 ? vndxIcon = 1 : vndxIcon++);
      InvalidateRect( vhwndIconLine, NULL, TRUE );
      UpdateWindow( vhwndIconLine );
      }
      break;

    case WM_CLOSE:
      //----------------------------------------------------------------//
      // Select... dialog is being dismissed, but hang-up just in case  //
      // a connection was in progress and kill the dial timer           //
      //----------------------------------------------------------------//
      HangUp(vhRasConn);
      if(vtimerDial) KillTimer( hdlg, vtimerDial );
      EndDialog( hdlg, TRUE ) ;
      DestroyWindow( hdlg );
      vhwndConnect = 0;
  }
  return FALSE;
}

/**********************************************************************/
/*                                                                    */
/* Function Name: GroupWinProc                                        */
/*                                                                    */
/* Description: Subclassed window procedure for group box controls.   */
/*                                                                    */
/* Input Parameters:                                                  */
/*                                                                    */
/*   hdlg     : Window handle.                                        */
/*   iMessage : Message identifier.                                   */
/*   wParam   : 16 bits of additional message dependent               */
/*              information.                                          */
/*   lParam   : 32 bits of additional message dependent               */
/*              information.                                          */
/*                                                                    */
/* InOut Parameters:                                                  */
/*                                                                    */
/*   None.                                                            */
/*                                                                    */
/* Output Parameters:                                                 */
/*                                                                    */
/*   BOOL     : dependent on message type                             */
/*                                                                    */
/**********************************************************************/
LRESULT __export CALLBACK GroupWinProc( HWND hwnd, UINT msg,
                                        WPARAM wParam, LPARAM lParam )
{
  LRESULT     lRetFlag = 0L;                // return flag to caller
  PAINTSTRUCT ps;                           // paint structure
  HPEN        hBlackPen;                    // black pen handle
  HPEN        hWhitePen;                    // white pen handle
  HPEN        hOldPen;                      // old pen handle
  HFONT       hFont;                        // font handle
  HFONT       hOldFont = 0;                 // old font handle
  RECT        rcClient;                     // window rectangle
  RECT        rect;                         // window update rectangle
  char        szBuf[CCHBUFFER];             // buffer for window text
  RECT        rectGroup;                    // groupbox rectangle

  switch ( msg )
  {
    case WM_PAINT:

      hFont = (HFONT)SendMessage( hwnd, WM_GETFONT, 0, 0L );
      BeginPaint( hwnd, &ps );
      GetClientRect( hwnd, &rcClient );

      FillRect( ps.hdc, &rcClient, vhbrushDlgBkgnd );

      hBlackPen = GetStockObject( BLACK_PEN );
      hWhitePen = GetStockObject( WHITE_PEN );
      CopyRect ( &rect, &rcClient );

      hOldPen = SelectObject( ps.hdc, hBlackPen );
      CopyRect ( &rect, &rcClient );
      MoveTo( ps.hdc, rect.left, rect.bottom );
      LineTo( ps.hdc, rect.left, rect.top );
        LineTo( ps.hdc, rect.right, rect.top );
      SelectObject( ps.hdc, hWhitePen );
      LineTo( ps.hdc, rect.right, rect.bottom );
      LineTo( ps.hdc, rect.left, rect.bottom );
      SelectObject( ps.hdc, hOldPen );

      if( hwnd == vhwndGroup1 )
      {
        GetWindowRect( hwnd, &rectGroup );

        /* Result listbox */
        GetWindowRect( vhwndRASList, &rect );
        rect.left = rect.left - rectGroup.left;
        rect.right = rect.right - rectGroup.left;
        rect.top = rect.top - rectGroup.top;
        rect.bottom = rect.bottom - rectGroup.top;
        InflateRect( &rect, 1, 1 );
        hOldPen = SelectObject( ps.hdc, vhpenEditTopBorder );
        MoveTo( ps.hdc, rect.left, rect.bottom );
        LineTo( ps.hdc, rect.left, rect.top );
        LineTo( ps.hdc, rect.right, rect.top );
        SelectObject( ps.hdc, hWhitePen );
        MoveTo( ps.hdc, rect.right-1, rect.top+1 );
        LineTo( ps.hdc, rect.right-1, rect.bottom-1 );
        LineTo( ps.hdc, rect.left, rect.bottom-1 );
        SelectObject( ps.hdc, hOldPen );
      }

      /* if groupbox control contains text - draw it using correct font */
      GetWindowText( hwnd, szBuf, CCHBUFFER );
      if( strlen(szBuf) )
      {
        rect.left += GROUP_BORDER;
        rect.top += GROUP_BORDER;
        SetBkMode( ps.hdc, TRANSPARENT );
        if( hFont )
          hOldFont = SelectObject( ps.hdc, hFont );

        DrawText( ps.hdc, szBuf, strlen( szBuf ), &rect, DT_SINGLELINE );

        if( hOldFont )
          SelectObject( ps.hdc, hOldFont );
      }

      EndPaint( hwnd, &ps );
      break;

    case WM_CLOSE:
      DestroyWindow( hwnd );
      break;

    default:
      lRetFlag = vDefGroupWinProc( hwnd, msg, wParam, lParam );
      break;

  }
  return lRetFlag;
}

/**********************************************************************/
/*                                                                    */
/* Function Name: InfoWinProc                                         */
/*                                                                    */
/* Description: Window procedure for the InfoWin class.               */
/*                                                                    */
/* Input Parameters:                                                  */
/*                                                                    */
/*   hdlg     : Window handle.                                        */
/*   msg      : Message identifier.                                   */
/*   wParam   : 16 bits of additional message dependent               */
/*              information.                                          */
/*   lParam   : 32 bits of additional message dependent               */
/*              information.                                          */
/*                                                                    */
/* InOut Parameters:                                                  */
/*                                                                    */
/*   None.                                                            */
/*                                                                    */
/* Output Parameters:                                                 */
/*                                                                    */
/*   LRESULT  : dependent on message type                             */
/*                                                                    */
/**********************************************************************/
LRESULT CALLBACK _export  InfoWinProc( HWND hwnd, UINT msg,
                                       WPARAM wParam, LPARAM lParam )
{
  LRESULT     lRetFlag = 0L;                // Return flag to caller
  PAINTSTRUCT ps;                           // Paint structure
  HPEN        hBlackPen;                    // Black pen handle
  HPEN        hWhitePen;                    // White pen handle
  HPEN        hOldPen;                      // Old pen handle
  HFONT       hFont = 0;                    // Font handle
  HFONT       hOldFont = 0;                 // Old font handle
  RECT        rcClient;                     // Window rectangle
  RECT        rect;                         // Window update rectangle
  RECT        rectText;                     // Text rectangle
  char        szBuf[CCHBUFFER];             // Buffer for window text

  switch ( msg )
  {
    case WM_CREATE:

     // create suitable font to ensure complete message can be seen
     // in the infoline line, without the need for scrolling

      hFont = CreateFont( 8, 0, 0, 0, 0/*style*/, 0, 0, 0, 0, 0, 0, 0, 0, "MS Sans Serif" );
      SetWindowLong( hwnd, 0, (LONG)hFont );
      break;

    case WM_DESTROY:

      // get the font handle from window words and delete the font
      // before destroying the window

      hFont = (HFONT)GetWindowLong( hwnd, 0 );
      if( hFont )
        DeleteObject(hFont);
      break;

    case WM_PAINT:
                                // paint the info window (box + icon)
      BeginPaint( hwnd, &ps );
      GetClientRect( hwnd, &rcClient );

         // fill the whole area with gray */

      FillRect( ps.hdc, &rcClient, vhbrushDlgBkgnd );

         // draw the MQMAN icon on the right hand side */

      CopyRect ( &rect, &rcClient );

      rect.left = rect.right + INFOLINE_BORDER;
      rect.top += INFOLINE_BORDER;

         // draw an indent frame around the text area */

      hBlackPen = GetStockObject( BLACK_PEN );
      hWhitePen = GetStockObject( WHITE_PEN );
      CopyRect ( &rect, &rcClient );
      rect.left += INFOLINE_BORDER;
      rect.right -= INFOLINE_BORDER;
      rect.top += INFOLINE_BORDER;
      rect.bottom -= INFOLINE_BORDER;
      hOldPen = SelectObject( ps.hdc, hBlackPen );
      MoveTo( ps.hdc, rect.left, rect.bottom );
      LineTo( ps.hdc, rect.left, rect.top );
      LineTo( ps.hdc, rect.right, rect.top );
      SelectObject( ps.hdc, hWhitePen );
      LineTo( ps.hdc, rect.right, rect.bottom );
      LineTo( ps.hdc, rect.left, rect.bottom );
      SelectObject( ps.hdc, hOldPen );

         // draw the text in the text area

      GetWindowText( hwnd, szBuf, CCHBUFFER );
      SetBkMode( ps.hdc, TRANSPARENT );
      rect.left += INFOLINE_BORDER;
      hFont = (HFONT)GetWindowLong( hwnd, 0 );
      if( hFont )
        hOldFont = SelectObject( ps.hdc, hFont );
      CopyRect( &rectText, &rect );
      DrawText( ps.hdc, szBuf, strlen( szBuf ), &rectText,
                DT_CALCRECT | DT_VCENTER | DT_WORDBREAK);
      if (rectText.bottom > rect.bottom) rectText.bottom = rect.bottom;
      rect.top +=
        ( (rect.bottom - rect.top) - (rectText.bottom - rectText.top) ) / 2;
      rect.bottom -=
        ( (rect.bottom - rect.top) - (rectText.bottom - rectText.top) ) / 2;
      DrawText( ps.hdc, szBuf, strlen( szBuf ), &rect,
                DT_VCENTER | DT_WORDBREAK);
      if (hOldFont) SelectObject( ps.hdc, hOldFont );

      EndPaint( hwnd, &ps );
      break;

    case WM_CLOSE:
      DestroyWindow( hwnd );
      break;

    default:
      /* for all other messages, call the default window procedure */
      lRetFlag = DefWindowProc( hwnd, msg, wParam, lParam );
      break;
  }
  return lRetFlag;
}

/**********************************************************************/
/*                                                                    */
/* Function Name: IconWinProc                                         */
/*                                                                    */
/* Description: Window procedure for the IconWin class.               */
/*                                                                    */
/* Input Parameters:                                                  */
/*                                                                    */
/*   hdlg     : Window handle.                                        */
/*   msg      : Message identifier.                                   */
/*   wParam   : 16 bits of additional message dependent               */
/*              information.                                          */
/*   lParam   : 32 bits of additional message dependent               */
/*              information.                                          */
/*                                                                    */
/* InOut Parameters:                                                  */
/*                                                                    */
/*   None.                                                            */
/*                                                                    */
/* Output Parameters:                                                 */
/*                                                                    */
/*   LRESULT  : dependent on message type                             */
/*                                                                    */
/**********************************************************************/
LRESULT CALLBACK _export  IconWinProc( HWND hwnd, UINT msg,
                                       WPARAM wParam, LPARAM lParam )
{
  LRESULT     lRetFlag = 0L;                /* Return flag to caller  */
  PAINTSTRUCT ps;                           /* Paint structure        */
  RECT        rcClient;                     /* Window rectangle       */
  RECT        rect;                         /* Window update rectangle*/


  switch ( msg )
  {
    case WM_CREATE:

    case WM_DESTROY:
      break;

    case WM_ERASEBKGND:         // NO OP , to stop 'flicker'
      return TRUE;
      break;

    case WM_PAINT:
      // paint the info window (box + icon)

      BeginPaint( hwnd, &ps );
      GetClientRect( hwnd, &rcClient );

      // fill the whole area with gray

       FillRect( ps.hdc, &rcClient, vhbrushDlgBkgnd );

      // draw the icon on the right hand side

      CopyRect ( &rect, &rcClient );
      DrawIcon( ps.hdc, rect.left, rect.top, vahIcon[vndxIcon] );
      EndPaint( hwnd, &ps );
      break;

    case WM_CLOSE:
      DestroyWindow( hwnd );
      break;

    default:
      // for all other messages, call the default window procedure
      lRetFlag = DefWindowProc( hwnd, msg, wParam, lParam );
      break;
  }
  return lRetFlag;
}

/**********************************************************************/
/*                                                                    */
/* Function Name: SetInfoText                                         */
/*                                                                    */
/* Description: Set the text for the information line window.         */
/*                                                                    */
/* Input Parameters:                                                  */
/*                                                                    */
/*   hwndParent  : parent Window handle.                              */
/*   pszInfoText : pointer to new information line text.              */
/*                                                                    */
/* InOut Parameters:                                                  */
/*                                                                    */
/*   None.                                                            */
/*                                                                    */
/* Output Parameters:                                                 */
/*                                                                    */
/*   None.                                                            */
/*                                                                    */
/* Returns:                                                           */
/*                                                                    */
/*   BOOL     : Return TRUE if successful, FALSE otherwise            */
/*                                                                    */
/**********************************************************************/
BOOL SetInfoText( HWND hwndParent, LPSTR pszInfoText )
{
  HWND hwndInfo;                        // InfoLine window handle
  char szText[INFO_SIZE];               // Text buffer

  // get the existing window handle and associated text

  hwndInfo = GetDlgItem( hwndParent, IDC_INFOLINE );
  GetWindowText( hwndInfo, szText, sizeof(szText) );

  // if the new text differs from the existing text, update the
  // window - this prevents unnecessary paints

  if( strcmp( szText, pszInfoText ) )
  {
    SetWindowText( hwndInfo, pszInfoText );
    InvalidateRect( hwndInfo, NULL, TRUE );
    UpdateWindow( hwndInfo );
  }
  return TRUE;
}
